from requests import Response
from requests.models import HTTPError


class AlmaError(Exception):
    def __init__(self, response):
        alma_error = response.json()
        errors = alma_error.get("errorList", {}).get("error", [])
        message = ", ".join(
            "{} - {}".format(x["errorCode"], x["errorMessage"]) for x in errors
        )
        super().__init__(message)


class AlmaUserNotFound(AlmaError):
    pass


class AlmaCourseNotFound(AlmaError):
    pass


class AlmaReadingListNotFound(AlmaError):
    pass


class AlmaNotFound(Exception):
    """The search yielded no results"""


class AlmaTooManyResults(Exception):
    """The search yielded too many results"""


ERROR_CODES = {
    "401861": AlmaUserNotFound,  # User with identifier X was not found.
    "401890": AlmaUserNotFound,  # User with identifier X of type T was not found.
    "401914": AlmaCourseNotFound,
    "401915": AlmaReadingListNotFound
}


def handle_error(response: Response, err: HTTPError) -> None:
    error_payload = response.json()
    errors = error_payload.get("errorList", {}).get("error", [])
    for x in errors:
        error_code = x["errorCode"]
        if error_code in ERROR_CODES:
            error_cls = ERROR_CODES[error_code]
            raise error_cls(response) from err
    if errors:
        raise AlmaError(response) from err
    raise err
